@@ -11,9 +11,10 @@
 //
 
 //-- user query params
-var user_query = {};
-var user_format = null; //-- save format request, e.g. for motion-charts
-var qinfo       = null; //-- query info, set e.g. by html mode
+var user_query   = {};
+var user_format  = null; //-- save format request, e.g. for motion-charts
+var qinfo        = null; //-- query info, set e.g. by html mode
+var responseData = null; //-- cached response data, for browser-friendly 'save as'
 
 //-- timing
 var ttk_elapsed = 0;
@@ -37,6 +38,7 @@ var dcp_params_default = {
     "format" : "text",
     "debug" : 0,
     "groupby": null,
+    "eps" : 0
 };
 var dynformats = {
     "gmotion" : true,
@@ -101,22 +103,39 @@ function dqReady() {
     var uhref  = dcp_url_base + "?" + uparam;
     $("#rawLink").prop("href",uhref).text(uhref).show();
 
-    //-- send request
-    $.ajax({
-	type: "GET",
-	url: dcp_url_local+'?'+uparam,
-	dataType: "text",
-	success: dcpOnSuccess,
-	error: dcpOnError,
-	complete: dcpOnComplete
-    })
+    //-- check for pre-fetched data (--> loading from local file exported by browser "Save As" function)
+    if (dcpData != null) {
+	$("#profileDataD3").hide();
+	$("#dqForm td label").addClass("disabled");
+	$("#dqForm").on('submit',function() {
+	    alert("You cannot submit queries from an offline data set!");
+	    return false;
+	});
+	document.title += " [exported]";
+	$(".headers h1").append(" [exported]");
+	//$("#dqForm, #dqForm input, #dqForm select").prop("disabled",true);
+
+	dcpOnComplete(null,"success");
+	dcpOnSuccess(dcpData,"success",{"responseText":dcpData});
+    } else {
+	//-- send request
+	$.ajax({
+	    type: "GET",
+	    url: dcp_url_local+'?'+uparam,
+	    dataType: "text",
+	    success: dcpOnSuccess,
+	    error: dcpOnError,
+	    complete: dcpOnComplete
+	})
+    }
 }
 
 //----------------------------------------------------------------------
 // jqStatusSelection = dcpStatusMsg(cls,msg)
 function dcpStatusMsg(cls,msg) {
     var st = $("#status");
-    st.attr("class",cls).find(".msg").text(msg);
+    st.on("click", function() { st.fadeOut(); });
+    st.attr("class","status "+cls).find(".msg").text(msg);
     return st;
 }
 
@@ -124,8 +143,21 @@ function dcpStatusMsg(cls,msg) {
 function dcpErrorMsg(msg,time) { return dcpStatusMsg("error",msg).fadeIn(Number(time)); }
 function dcpWarnMsg(msg,time) { return dcpStatusMsg("warning",msg).fadeIn(Number(time)); }
 function dcpInfoMsg(msg,time) { return dcpStatusMsg("info",msg).fadeIn(Number(time)); }
+function dcpHintMsg(msg,time) { return dcpStatusMsg("info hint",msg).fadeIn(Number(time)); }
 function dcpClearMsg(time) { return dcpStatusMsg("","").fadeOut(Number(time)); };
 
+function dcpShowPrefetchHint() {
+    if (dcpData) {
+	dcpInfoMsg("Cached result data: you will be unable to submit new queries.").fadeOut(5000);
+    }
+}
+
+function dcpCacheData(data) {
+    if (dcpData == null) {
+	$("#diacolloResponseData").text('dcpData = ' + JSON.stringify(data).replace(/</g,'\\u003c').replace(/>/g,'\\u003e') + ';');
+    }
+}
+
 //----------------------------------------------------------------------
 var isDiff;
 var isAbsDiff;
@@ -134,6 +166,12 @@ function dcpOnSuccess(data,textStatus,jqXHR) {
     isDiff    = Boolean(user_query.profile.match(/^diff-/));
     isAbsDiff = Boolean(isDiff && user_query.diff.match(/^adiff/)); //|min
 
+    //-- cache response if appropriate
+    if (dcpData==null) {
+	dcpCacheData(jqXHR.responseText);
+    }
+
+    //-- format dispatch
     if (user_format == "html" || jqXHR.responseText.match(/<html\b/i)) {
 	//-- response: html
 	dcpFormatHtml(data, jqXHR);
@@ -185,7 +223,7 @@ function dcpOnComplete(jqXHR, textStatus) {
 //----------------------------------------------------------------------
 function dcpFormatHtml(data, jqXHR) {
     //-- parse response
-    $("#profileDataHtml").append( $.parseHTML(jqXHR.responseText, document, true) );
+    $("#profileDataHtml").empty().append( $.parseHTML(jqXHR.responseText, document, true) );
     $("#profileDataHtml").find("table").addClass("dbViewTable dcpTable " + (isDiff ? "diffTable" : "prfTable"));
     if ($("#profileDataHtml td").size()==0) {
 	dcpErrorMsg("Error: no data to display!");
@@ -207,7 +245,7 @@ function dcpFormatHtml(data, jqXHR) {
     $("#profileDataHtml tr:not(:first-child)").each(function(i,tr) {
 	var label = $(tr).find(":nth-child("+(ilabel+1)+")").text();
 	if (label != plabel) {
-	    $(tr).addClass("newlabel");
+	    $(tr).addClass("newlabel").attr("id",label);
 	    plabel = label;
 	}
     });
@@ -264,6 +302,13 @@ function dcpFormatHtml(data, jqXHR) {
 	    st.css({"background-color":heatcolorv(val, min, max), width:sz, height:sz});
 	});
     }
+
+    //-- jump to fragment if specified
+    var fragment = locFragment(window.location);
+    if (fragment != "") {
+	window.location.hash = '';
+	window.location.hash = '#'+fragment;
+    }
 }
 
 //----------------------------------------------------------------------
@@ -335,6 +380,10 @@ function dcpFormatHiChart(data, jqXHR) {
     }
     dcpStatusMsg("loading","Rendering...");
 
+    //-- hichart: enable "download" icon
+    $("#d3icons > a").hide();
+    $("#profileDataD3, #d3icons, #exportBtn").fadeIn();
+
     //-- setup plot data
     var cdata = { //-- chart data
 	chart: {
@@ -462,12 +511,14 @@ var brushSnap;    //-- function variable for brush snap
 //  + array-valued item data keys are indexed by dlabels[] index
 //  + options:
 //     mode: MODE,   //-- parse mode (known values: "bubble", "cloud")
+var d3data = null;
 function dcpParseFlat(data,opts) {
     //-- status message
     dcpStatusMsg("loading","Parsing...");
 
     //-- parse JSON data
     if (!(data instanceof Object)) { data = $.parseJSON(data); }
+    d3data = data;
     qinfo = data.qinfo;
     if (data.profiles.length == 0) {
 	dcpErrorMsg("Error: no data to display!");
@@ -486,7 +537,7 @@ function dcpParseFlat(data,opts) {
     dlabels = [];
     itemid  = {};
     items   = [];
-    dcur = 0; //-- current subprofile index
+    dcur    = 0; //-- current subprofile index
 
     //-- get data range
     data.profiles.forEach(function (p) { p.range = d3.extent(d3.values(p[p.score])); });
@@ -623,6 +674,14 @@ function dcpParseFlat(data,opts) {
 	brushSnap   = dcpCloudSnap;
     }
 
+    //-- initialize current subprofile index (dcur) from URL fragment
+    var fragment = locFragment(window.location);
+    if (fragment != "" && dlabels.indexOf(fragment) >= 0) {
+	dcur = dlabels.indexOf(fragment);
+    } else {
+	dcur = 0;
+    }
+
     return data;
 }
 
@@ -817,9 +876,10 @@ function d3InfoPopupUpdate(snapto) {
 //    dbrush = d3brush(dlabels, selector, opts)
 // + sets global dbrush=brush, bhandle=handle, bsvg=brush-svg
 var dbrush, bhandle, bsvg;
-function dcpBrushSlider(dlabels, selector, opts) {
+function dcpTransportSlider(dlabels, parent_selector, opts) {
 
     //-- setup defaults
+    if (opts.id==null) opts.id = "d3slider";
     if (opts.width==null) opts.width = 800;
     if (opts.height==null) opts.height = 50;
     var margin = opts.margin==null ? {} : opts.margin;
@@ -827,6 +887,9 @@ function dcpBrushSlider(dlabels, selector, opts) {
     if (margin.right==null) margin.right = 0;
     if (margin.top==null) margin.top = 0;
     if (margin.bottom==null) margin.bottom = 0;
+    if (opts.translate==null) opts.translate = {x:0,y:0};
+
+    //debug_log("dcpTransportSlider(dlabels="+JSON.stringify(dlabels)+", parent_selector="+JSON.stringify(parent_selector)+", opts="+JSON.stringify(opts)+")");
 
     //-- common variables
     var width  = opts.width - margin.left - margin.right;
@@ -838,22 +901,22 @@ function dcpBrushSlider(dlabels, selector, opts) {
 
     var brush = dbrush = d3.svg.brush()		//d3.v4: d3.brushX() 
 	.x(xscale)				//d3.v4: ???
-	.extent([0, 0])
+	.extent([dcur, dcur])
 	.on("brushstart", dcpOnBrushStart)	//d3.v4: ???
 	.on("brush", dcpOnBrush)		//d3.v4: ???
 	.on("brushend", dcpOnBrushEnd)		//d3.v4: ???
     ;
 
-
-    var svg = bsvg = d3.select(selector)
-	.append("svg")
+    var parent = d3.select(parent_selector);
+    var svg = bsvg = parent.append("g")
+	.attr("id", opts.id)
 	.attr("width", opts.width)
 	.attr("height", opts.height)
 	.append("g")
-	.attr("class", "brush")
-	.attr("transform", "translate(" + margin.left + "," + margin.top + ")");
+	.attr("class","brush")
+    	.attr("transform", "translate(" + (margin.left+opts.translate.x) + "," + (margin.top+opts.translate.y) + ")");
     svg.append("title")
-	.text("Date-slice to display (drag, left/right arrow, Home, End)");
+    	.text("Date-slice to display (drag, left/right arrow, Home, End)");
 
     var labelPad = 6;
     var dy = 15+labelPad; //height/2;
@@ -917,7 +980,7 @@ function dcpBrushSlider(dlabels, selector, opts) {
 	.attr("y",10)
 	.text("");
 
-    var x0 = 0; //xscale(dlabels[0]);
+    var x0 = dcur; //xscale(dlabels[0]);
     slider
 	.call(brush.event)		//-- d3.v4: ???
 	.call(brush.extent([x0,x0]))
@@ -1025,7 +1088,7 @@ function dcpBrushKeys(sel2tabindex) {
 	var jsel = $(sel);
 	jsel.attr("tabindex",ix)
 	    .keydown(function(e) {
-		debug_log("keydown: which="+e.which+"; key="+e.key);
+		//debug_log("keydown: which="+e.which+"; key="+e.key);
 		var dskip = 0;
 		var dosnap = true;
 		switch(e.which) {
@@ -1037,7 +1100,7 @@ function dcpBrushKeys(sel2tabindex) {
 
 		case 83: // s(Save)
 		case 88: // x(eXport)
-		    document.getElementById("export").click();
+		    document.getElementById("exportBtn").click();
 		    break;
 
 		case 35: // end
@@ -1143,6 +1206,7 @@ function d3SetFocus(val) {
     if (val==null) val=!d3HasFocus;
     d3HasFocus = val;
     //debug_log("setFocus("+val+")");
+    //exportMenuHide(0);
     if (val) {
 	//-- enable keyboard focus
 	$("#kbicon").attr("title","Keyboard shortcuts enabled (arrow-keys, spacebar)");
@@ -1158,8 +1222,9 @@ function d3SetFocus(val) {
 // d3: play/pause transport
 var dcpPlaying=false;
 var dcpSpeed=1, speedNode, speedBrush, speedScale, speedHandle;
-function dcpTransport(selector, opts) {
+function dcpTransportButtons(parent_selector, opts) {
     if (opts==null) { opts = {}; }
+    if (opts.id==null) { opts.id = "d3buttons"; }
     if (opts.width==null) { opts.width = 75; }
     if (opts.height==null) { opts.height = 50; }
     if (opts.pad==null)  { opts.pad = 10; }
@@ -1170,9 +1235,14 @@ function dcpTransport(selector, opts) {
     if (margin.right==null) { margin.right=1; }
     if (opts.bpad==null) { opts.bpad = (opts.height-margin.top-margin.bottom)/8; }
     if (opts.bround==null) { opts.bround = (opts.height-margin.top-margin.bottom)/4; }
-    var div = d3.select(selector);
 
-    var svg = d3.select(selector).append("svg")
+    //debug_log("dcpTransportButtons(parent_selector="+JSON.stringify(parent_selector)+", opts="+JSON.stringify(opts)+")");
+    
+    //var div = d3.select(selector);
+    var parent = d3.select(parent_selector);
+
+    var svg = parent.append("g")
+	.attr("id", opts.id)
 	.attr("width", opts.width)
 	.attr("height", opts.height);
     var btn = svg.append("g")
@@ -1191,6 +1261,7 @@ function dcpTransport(selector, opts) {
 	.attr("y",0)
 	.attr("width",bsize)
 	.attr("height",bsize)
+        .attr("rx",opts.bround)
 	.attr("ry",opts.bround);
 
     var bplay = btn.append("polygon")
@@ -1307,7 +1378,8 @@ function dcpTransport(selector, opts) {
 }
 
 function d3SpeedBrushDebug(label) {
-    debug_log(label+": val=" + speedBrush.extent()[0] + " ~ " + speedScale(speedBrush.extent()[0]));
+    //debug_log(label+": val=" + speedBrush.extent()[0] + " ~ " + speedScale(speedBrush.extent()[0]));
+    return;
 }
 
 function d3SpeedBrushMove(value,dur,easeby) {
@@ -1359,6 +1431,7 @@ function d3OnSpeedBrushEnd() {
 function dcpPlay(playing,force) {
     if (playing==null) playing = dcpPlaying;
     if (force==null) force=false;
+    //exportMenuHide(0);
 
     //-- setup buttons
     var btn = d3.selectAll(".btn");
@@ -1403,9 +1476,10 @@ function dcpPlay(playing,force) {
 
 //----------------------------------------------------------------------
 // d3: legend (color-scale / "y axis")
-function dcpLegend(selector,opts) {
+function dcpLegend(parent_selector,opts) {
     //-- defaults
     if (opts==null) opts={};
+    if (opts.id==null) { opts.id="d3legend"; }
     if (opts.width==null) { opts.width = 50; }
     if (opts.height==null) { opts.height = 450; }
     if (opts.opacity==null) { opts.opacity = 1; }
@@ -1414,6 +1488,9 @@ function dcpLegend(selector,opts) {
     if (margin.right==null) margin.right = 0;
     if (margin.top==null) margin.top = 0;
     if (margin.bottom==null) margin.bottom = 0;
+    if (opts.translate==null) opts.translate = {x:0,y:0};
+
+    //debug_log("dcpLegend(parent_selector="+JSON.stringify(parent_selector)+", opts="+JSON.stringify(opts)+")");
 
     //-- common variables
     var width  = opts.width - margin.left - margin.right;
@@ -1423,18 +1500,20 @@ function dcpLegend(selector,opts) {
 	.range([0,height])
 	.clamp(true);
 
-    var svg = d3.select(selector)
+    var parent = d3.select(parent_selector);
+
+    var svg = parent.append("g")
+	.attr("id",opts.id)
 	.classed("d3legend",true)
-	.attr("title","Node colors by collocate score"+(isDiff ? " difference" : ""))
-	.style("width",opts.width+"px")
-	.style("height",opts.height+"px")
-	.append("svg")
-	.attr("class","scale")
-	.attr("width",width)
-	.attr("height",opts.height);
+	.classed("scale",true)
+	.attr("width", opts.width)
+	.attr("height", opts.height)
+    	.attr("transform","translate("+(margin.left+opts.translate.x)+","+(margin.top+opts.translate.y)+")");
+    svg.append("title")
+	.text("Legend: node colors by collocate score"+(isDiff ? " difference" : ""));
+
     svg.append("g")
 	.attr("class", "y axis")
-	.attr("transform","translate("+margin.left+","+margin.top+")")
 	.call(d3.svg.axis()
 	      .scale(yscale)
 	      .orient("right")
@@ -1495,17 +1574,29 @@ function dcpLegend(selector,opts) {
 }
 
 //----------------------------------------------------------------------
-// d3: export
+// d3: export: export svg guts
 //  + see http://stackoverflow.com/questions/23218174/how-do-i-save-export-an-svg-file-after-creating-an-svg-with-d3-js-ie-safari-an
-function d3export() {
+function d3exportSvg(event) {
     //get svg element.
-    var svg = d3.select(".d3chart svg").node().cloneNode(true);
+    var mode = $("#profileDataChart").is(":visible") ? 'hichart' : 'd3';
+    var svg;
+    if (mode == 'hichart') {
+	svg = d3.select("#profileDataChart svg");
+    } else {
+	svg = d3.select("#d3content");
+    }
+    svg = svg.node().cloneNode(true);
 
-    //-- insert inline css into svg
-    var style = document.createElementNS("http://www.w3.org/2000/svg","style");
-    var css   = getcss(true,/diacollo\.css/,/d3/);
-    $(style).text(css);
-    svg.insertBefore(style, svg.firstChild);
+    //-- insert inline css into svg (d3 only)
+    if (mode == 'd3') {
+	var style = document.createElementNS("http://www.w3.org/2000/svg","style");
+	var css   = getcss(true,/diacollo\.css/,/d3/);
+	$(style).text(css);
+	svg.insertBefore(style, svg.firstChild);
+
+	//-- boldface current slider slice label (hack for inkscape)
+	$(svg).find("#d3slider .brush .tick text.selected").css("font-weight","bold");
+    }
 
     //-- get svg source.
     var serializer = new XMLSerializer();
@@ -1527,12 +1618,18 @@ function d3export() {
     var url = "data:image/svg+xml;charset=utf-8,"+encodeURIComponent(source);
 
     //-- set url value to the export button's element's href attribute
-    $("#export")
+    $("#exportBtn") 
 	//.prop("target","_tab")
 	.prop("download","diacollo.svg")
 	.prop("href",url);
 
-    //-- propagte event (we were called by clicking on the link whose href we just set)
+    //-- go get it (must use DOM click() method, not jQuery if using exportTarget != event.target)
+    /*
+      exportMenuHide();
+      document.getElementById('exportTarget').click();
+    */
+
+    //-- direct-click button: just return true
     return true;
 }
 
@@ -1541,7 +1638,6 @@ function d3export() {
 //  + see http://stackoverflow.com/questions/1679507/getting-all-css-used-in-html-file
 function getcss(wantIntern, hrefRegex, selectorRegex) {
     var css = ""; //variable to hold all the css that we extract
-
     //-- add internal styles
     if (wantIntern) {
 	$("style").each(function(i,s) {
@@ -1571,7 +1667,7 @@ function getcss(wantIntern, hrefRegex, selectorRegex) {
 // d3: common: controls & geometry
 //  + opts:
 //     legendOpacity:OPACITY  //-- legend color-scale opacity
-//     width:WIDTH,           //-- total width (default=$("#profileDataD3").width())
+//     width:WIDTH,           //-- total width (default=window.innerWidth-16)
 //     height:HEIGHT,         //-- total height (default=500)
 //     bodyWidth:WIDTH        //-- body,slider width
 //     buttonsWidth:WIDTH     //-- buttons width
@@ -1585,34 +1681,77 @@ function d3SetupCommon(opts) {
     if (opts.legendOpacity==null) opts.legendOpacity=1;
 
     //-- common variables
-    $("#profileDataD3").fadeIn();
     if (opts==null) opts={};
-    if (opts.width==null) opts.width   = $("#profileDataD3").width();
     if (opts.height==null) opts.height = 500;
-    if (opts.buttonsWidth==null) opts.buttonsWidth = $("#d3buttons").width();
-    if (opts.sliderWidth==null) opts.sliderWidth = opts.width - opts.buttonsWidth - 20;  //-- 10px goofiness margin (firefox, mobile emulation)
-    if (opts.legendWidth==null) opts.legendWidth = $("#d3legend").width();
-    if (opts.bodyWidth==null) opts.bodyWidth = opts.width-opts.legendWidth - 20; //-- 10px goofiness margin (firefox, mobile emulation)
-    if (opts.bodyHeight==null) opts.bodyHeight = opts.height - 50;              //-- +1 for border
+    if (opts.buttonsWidth==null) opts.buttonsWidth = 90; //$("#d3buttons").width();      //-- not defined yet!
+    if (opts.legendWidth==null) opts.legendWidth = 50; //$("#d3legend").width();         //-- not defined yet!
+    if (opts.bodyHeight==null) opts.bodyHeight = opts.height - 50 - 5;                   //-- -5:ypad
 
-    debug_log("d3SetupCommon: "+JSON.stringify(opts));
+    //debug_log("d3SetupCommon:pre: "+JSON.stringify(opts));
 
-    //-- date slider (d3 svg.brush)
-    $("#profileDataD3").fadeIn();
-    var bpadx = 50; //-- brush x-padding
-    dcpBrushSlider(dlabels, "#d3slider", {width:opts.sliderWidth, height:50, margin:{top:0, bottom:0, left:bpadx, right:bpadx}});
-    dcpTransport("#d3transport #d3buttons", {width:opts.buttonsWidth, height:50});
+    //-- cleanup any stale content & create new svg with proper height (may add scrollbars!)
+    $("#d3content").remove();
+    var svg = d3.select("#profileDataD3")
+	.append("svg")
+	.attr("id","d3content")
+	//.attr("width",opts.width) //-- delayed in case scrollbars were added
+	.attr("height", opts.height);
 
-    //-- legend
-    dcpLegend("#d3legend", {width:opts.legendWidth, height:opts.bodyHeight, margin:{top:5,bottom:5,left:0,right:0}, opacity:opts.legendOpacity});
+
+    //-- get width (temporary show)
+    $("#profileDataD3").show();
+    if (opts.width==null) opts.width = $(".outer").width();
+    if (opts.sliderWidth==null) opts.sliderWidth = opts.width - opts.buttonsWidth ;//- 20;  //-- 10px goofiness margin (firefox, mobile emulation)
+    if (opts.bodyWidth==null) opts.bodyWidth = opts.width-opts.legendWidth ;//- 20;         //-- 10px goofiness margin (firefox, mobile emulation)
+    $("#profileDataD3").hide();
+    svg.attr("width",opts.width);
+
+    //debug_log("d3SetupCommon+width: "+JSON.stringify(opts));
+
+    //-- transport: buttons + brush-slider
+    var transport = svg.append("g")
+	.attr("id","d3transport")
+	.attr("width",opts.width)
+	.attr("height",50);
+
+    var sliderPadX = 50;
+    dcpTransportButtons("#d3transport", {id:"d3buttons", width:opts.buttonsWidth, height:50});
+    dcpTransportSlider(dlabels, "#d3transport", { id:"d3slider", width:opts.sliderWidth, height:50,
+						  margin:{top:0, bottom:0, left:sliderPadX, right:sliderPadX},
+						  translate:{x:opts.buttonsWidth, y:0}
+						});
+
+    //-- legend + body
+    dcpLegend("#d3content", { id:"d3legend", width:opts.legendWidth, height:opts.bodyHeight, opacity:opts.legendOpacity,
+			      margin:{top:10,bottom:10,left:0,right:0},
+			      translate:{x:0,y:50},
+			    });
+    var main = svg.append("g")
+	.attr("id","d3main")
+	.attr("transform", "translate("+(opts.legendWidth)+","+55+")")
+    ;
+    var bg = main.append("rect")
+	.attr("id","d3background")
+        .attr("width",opts.bodyWidth)
+	.attr("height",opts.bodyHeight)
+    ;
+    var body = main.append("g")
+	.attr("id","d3body")
+	.attr("width",opts.bodyWidth)
+	.attr("height",opts.bodyHeight)
+    ;
+    var frame = main.append("rect")
+	.attr("id", "d3frame")
+	.attr("rx",$("#d3buttons .btn .border").attr("rx"))
+	.attr("ry",$("#d3buttons .btn .border").attr("ry"))
+        .attr("width",opts.bodyWidth-1)
+	.attr("height",opts.bodyHeight-1)
+    ;
 
     //-- key bindings
     dcpBrushKeys({".content":1});
     $("#d3icons").fadeIn();
 
-    //-- export
-    $("#export").on("click",d3export).show();
-
     //-- status
     $("#statusRel").addClass("d3");
     dcpStatusMsg("loading","Rendering...");
@@ -1621,7 +1760,7 @@ function d3SetupCommon(opts) {
 }
 
 //----------------------------------------------------------------------
-// d3: bubble: force-directed graph chart;
+// d3: bubble: force-drirected graph chart;
 // + see https://githu    //-- common variablesb.com/mbostock/d3/wiki/Force-Layout (force layout demo)
 // + see https://gist.github.com/mbostock/3231298 (collision detection demo)
 var bubbleOpacity = 0.9;
@@ -1636,13 +1775,11 @@ function dcpFormatBubble(data, jqXHR) {
     //-- setup d3 force chart
     var psvg = d3.select("#d3body")
 	.classed("d3chart",true)
-	.attr("title","Bubble-graph for selected date-slice. Click+drag to move bubbles, double-click to display details.")
-	.style("width",width+"px")
-	.style("height",height+"px")
-	.append("svg")
-	.attr("width", width)
-	.attr("height", height)
-	.classed("d3force",true);
+    	.classed("d3force",true)
+    ;
+    d3.select("#d3main")
+	.append("title")
+	.text("Bubble-graph for selected date-slice. Click+drag to move bubbles, double-click to display details.");
 
     var force = dforce = d3.layout.force()
 	.links([])
@@ -1664,12 +1801,16 @@ function dcpFormatBubble(data, jqXHR) {
 		});
 	})
     ;
+    
     dcpForceSnap();
+    //dcpBrushMove(dcur);
 
     //dcpErrorMsg("-- work in progress --");
     dcpClearMsg();
     $(".rawURL").hide();
     $("#profileDataD3").fadeIn();
+
+    //dcpShowPrefetchHint(); //-- show "pre-fetched data" hint if appropriate
     return;
 }
 
@@ -1685,6 +1826,7 @@ function dcpForceSnap(value) {
 
 //--------------------------------------------------------------
 // d3: bubble: force layout update (interp)
+var d3BodySelector = '#d3body';
 function dcpForceInterp(value,dur,easeby) {
     if (dforce==null) return; //-- not defined yet
     if (value==null) { value = dcur; }
@@ -1700,7 +1842,7 @@ function dcpForceInterp(value,dur,easeby) {
     ditems.forEach(function(i) { i.gravity = linterp(i.value,dcur); });
     dforce.nodes(ditems);
 
-    var psvg = d3.select("svg.d3force");
+    var psvg = d3.select(d3BodySelector);
     var nodes = psvg.selectAll(".node")
 	.data(ditems, function(d) { return d.id; });
 
@@ -1831,12 +1973,11 @@ function dcpFormatCloud(data, jqXHR) {
     //-- setup cloud svg item
     var psvg = d3.select("#d3body")
 	.classed("d3chart",true)
-	.attr("title","Tag-cloud for selected date-slice. Click a word to display details.")
-    //.style("background","#333")
-	.append("svg")
-	.attr("width", width)
-	.attr("height", height)
-	.classed("d3cloud",true);
+	.classed("d3cloud",true)
+    ;
+    d3.select("#d3main")
+	.append("title")
+	.text("Tag-cloud for selected date-slice. Click a word to display details.");
 
     //-- (re-)initialize cloud layout
     dcpCloudSetup(psvg);
@@ -1853,7 +1994,7 @@ xrandom = Math.random;
 var dcpCloudScale = 1;
 var dcpCloudSizeMax = null;
 function dcpCloudSetup(psvg) {
-    if (psvg==null)  { psvg=d3.select("svg.d3cloud"); }
+    if (psvg==null)  { psvg=d3.select(d3BodySelector); }
 
     //xrandom = Math.random;
     //seed = 42;
@@ -1873,11 +2014,12 @@ function dcpCloudSetup(psvg) {
     var rotations 
 	= [0];
 	//= [0,90];
-	//= [0,90,45,-45];
+        //= [0,90,45,-45];
 	//= [0,45,-45];
-	//= [45,-45];
+       //= [45,-45];
+    var main  = d3.select("#d3main");
     var cloud = dcloud = d3.layout.cloud()
-	.size([Number(psvg.attr("width")),Number(psvg.attr("height"))])
+	.size([Number(psvg.attr("width")),Number(psvg.attr("height"))+25]) //-- weird unused space at bottom: tweaking height+25
 	.words(items)
 	.padding(1)
 	.random(xrandom)
@@ -1927,7 +2069,7 @@ red4 = ['rgb(254,240,217)','rgb(253,204,138)','rgb(252,141,89)','rgb(215,48,31)'
 //--------------------------------------------------------------
 // d3: cloud: callbacks
 function dcpCloudEnd(placedWords,bounds) {
-    var svg = d3.select("svg.d3cloud");
+    var svg = d3.select(d3BodySelector);
     svg
 	.append("g")
 	.attr("transform", "translate(" + dcloud.size()[0]/2 + "," + dcloud.size()[1]/2 + ")");
@@ -1970,7 +2112,10 @@ function dcpCloudEnd(placedWords,bounds) {
     //dcpCloudDebug(0,'linear',0,true);  return; //-- we don't get collisions if we do dcpCloudDebug() before dcpCouldInterp(): why???
 
     //-- draw data
-    dcpCloudInterp(0,1000,"cubic-in-out");
+    dcpCloudInterp(dcur,1000,"cubic-in-out");
+    //dcpBrushMove(dcur,1000,"cubic-in-out");
+
+    //dcpShowPrefetchHint(); //-- show "pre-fetched data" hint if appropriate
 }
 
 //-- debug: bbox string for getBoundingClientRect()
@@ -2009,7 +2154,7 @@ function dcpCloudInterp(value,dur,easeby) {
 	};
     };
 
-    var nodes = d3.select("svg.d3cloud > g")
+    var nodes = d3.select(d3BodySelector+" > g")
 	.selectAll(".node")
 	.data(ditems, function(d) { return d.id; });
 
@@ -2090,7 +2235,7 @@ function dcpCloudDebug(dur,easeby,opacity,doInterp) {
 	};
     };
 
-    var nodes = d3.select("svg.d3cloud > g")
+    var nodes = d3.select(d3BodySelector+" > g")
 	.selectAll(".node")
 	.data(ditems, function(d) { return d.id; });
 
@@ -2297,6 +2442,19 @@ function escapeHTML(s) {
     });
 }
 
+//--------------------------------------------------------------
+function urlFragment(url) {
+    var a = document.createElement('a');
+    a.href = url;
+    var fragment = a.hash.replace(/^#/,'');
+    $(a).remove();
+    return fragment;
+}
+
+function locFragment(loc) {
+    return loc.hash.replace(/^#/,'');
+}
+
 //--------------------------------------------------------------
 function escapeDDC(s, enquote) {
     if (s.search(/\W/) == -1) { return s; }
